package pcoprs;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

//    Interface for Buffulo PC-OP-RS1
//              USB IR remote controller
//    ver. 0.1 2006/12/26  Shinji Kono kono@ie.u-ryukyu.ac.jp

//    Everyone is permitted to do anything on this program 
//    including copying, modifying, improving,
//    as long as you don't try to pretend that you wrote it.
//    i.e., the above copyright notice has to appear in all copies.  
//    Binary distribution requires original version messages.
//    You don't have to ask before copying, redistribution or publishing.
//    THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE.


public class RemoteCommander {
	String device;
	private File dev;
	private FileInputStream reader;
	private FileOutputStream writer;
	
	static final String version = "ver. 0.2 2006/12/26";
	static final byte [] LED = { 0x69 };
	static final byte [] RECEIVE = { 0x72 };
	static final byte [] SEND = { 0x74 };
	
	static final int OK = 0x4f;
	static final int ACK = 0x59;
	static final int START = 0x53;
	static final int CH1 = 0x31;
	static final int CH2 = 0x32;
	static final int CH3 = 0x33;
	static final int CH4 = 0x34;
	static final int END = 0x45;
	static final int DATASIZE = 240;
	byte [] buf1;
	
	public RemoteCommander(String device) throws IOException, InterruptedException {
		this.device = device;
		this.buf1 = new byte[1];
		pcoprs();
	}

	public synchronized void pcoprs() throws IOException,
			InterruptedException {
		String[] cmd = { "/bin/stty", "-f", device, "115200", "-icanon", "raw",
				"cs8",
				"-echo" };

		Process p = Runtime.getRuntime().exec(cmd);


		dev = new File(device);

		reader = new FileInputStream(dev);
		writer = new FileOutputStream(dev);
		wait(100);
		writer.write(LED);
		wait(100);
		reader.read(buf1);
	}
	
	public synchronized void wait100() throws InterruptedException {
		wait(100);
	}
	
	public void read() throws IOException {
		byte [] buf = new byte[DATASIZE+1];
		writer.write(RECEIVE);
		do {
			reader.read(buf1);
		} while(buf1[0]!=START);
		reader.read(buf);
		System.out.write(buf, 0, buf.length-1);
	}

	synchronized void write(String [] arg) throws IOException, InterruptedException {

		if (arg.length<3) {
			System.out.println("Usage: device write [1-4] < data");
			return ;
		}
		byte [] buf = new byte[DATASIZE];
		System.in.read(buf);
		wait(400);		
		writer.write(SEND);
		do {
			reader.read(buf1);
		} while(buf1[0]!=ACK);
		writer.write(arg[2].getBytes());
		do {
			reader.read(buf1);
		} while(buf1[0]!=ACK);
		writer.write(buf);
		System.err.println("sending "+buf.length);
		if(false) {
			wait(1000);
		}
		do {
			reader.read(buf1);
		} while(buf1[0]!=END);
		
		// System.err.write(arg[2].getBytes());
		// System.out.write(buf);
	}
	
	public synchronized void led() throws IOException, InterruptedException {
		writer.write(LED);
		wait(100);
		reader.read(buf1);
		System.out.write(buf1);
	}
	
	static public void main(String [] arg) throws IOException, InterruptedException {
		if (arg.length<2) {
			System.out.println("Usage: device command (read/write ch/led)");
			return;
		}
		RemoteCommander self = new RemoteCommander(arg[0]);
		System.err.println(arg[1]+" doing...");
		switch(arg[1].charAt(0)) {
		case 'r':
			self.read(); break;
		case 'w':
			self.write(arg); break;
		case 'l':
			self.led(); break;
		default:
			System.err.println("unknown "+arg[1]);
			return;
		}
		System.err.println();
	}

}
